searchNodes=[{"ref":"Ecto.Rescope.html","title":"Ecto.Rescope","type":"module","doc":"Rescopes the default query on an Ecto schema. An Ecto Schema is typically scoped by the Ecto.Schema.schema/2 macro, which defines the query as: def __schema__(:query) do %Ecto.Query{ from: %Ecto.Query.FromExpr{ source: {unquote(source), __MODULE__}, prefix: unquote(prefix) } } end This has a downside in that the schema cannot define a default scope for all queries to follow. For instance, perhaps one wants to ensure that soft-deleted records are not returned by default. To accomplish this, one might exclude any record with an is_deleted field set to true. def without_deleted(query) do from(q in query, where: q.is_deleted == false) end Usage By using the rescope/1 attribute or macro provided by Ecto.Rescope, one can override by passing a function that takes an Ecto.Query struct as the sole argument. This function must return an Ecto.Query struct. NOTE: The macro must be invoked after the Ecto.Schema.schema/2 macro unless using an attribute. NOTE: The @rescope attribute does not accept anonymous functions. Example use Ecto.Rescope @rescope &amp;ThisModule.without_deleted/1 schema &quot;user&quot; do field(:is_deleted, :boolean) end def without_deleted(query) do from(q in query, where: q.is_deleted == false) end Using the macro directly. Example import Ecto.Rescope schema &quot;user&quot; do field(:is_deleted, :boolean) end rescope(&amp;without_deleted/1) def without_deleted(query) do from(q in query, where: q.is_deleted == false) end At this point, any queries using the schema will now be defined with the new default scope."},{"ref":"Ecto.Rescope.html#rescope/1","title":"Ecto.Rescope.rescope/1","type":"macro","doc":"Resets the default query on the Ecto schema. Accepts a 1-arity function that takes and returns an Ecto.Query struct. In addition to redefining the default scope, the macro defines two utility functions: unscoped/0 and scoped/0. These are used in situations where the overridden scope is either undesirable, or caveats exist that prevent use of the rescoped query. See: README for caveats"},{"ref":"readme.html","title":"Ecto.Rescope","type":"extras","doc":"Ecto.Rescope Extends Ecto to allow rescoping of the default schema query. A typical usecase for this functionality is excluding soft-deleted records by default."},{"ref":"readme.html#usage","title":"Ecto.Rescope - Usage","type":"extras","doc":"The most basic example for rescoping is to add directly to a schema with an inline function. defmodule User do use Ecto.Schema import Ecto.Query, only: [from: 2] import Ecto.Rescope schema &quot;users&quot; do field(:is_deleted, :boolean) end rescope(fn query -&gt; from(q in query, where: q.is_deleted == false) end) end In a more abstracted example, we might move the logic to a separate module for reuse. defmodule SoftDelete do import Ecto.Query, only: [from: 2] def exclude_deleted(query) do from(q in query, where: q.is_deleted == false) end end defmodule User do use Ecto.Schema import Ecto.Rescope schema &quot;users&quot; do field(:is_deleted, :boolean) end rescope(&amp;SoftDelete.exclude_deleted/1) end It is possible to add a @rescope attribute to your schema with an external function. defmodule SoftDelete do import Ecto.Query, only: [from: 2] def exclude_deleted(query) do from(q in query, where: q.is_deleted == false) end end defmodule User do use Ecto.Schema use Ecto.Rescope @rescope &amp;SoftDelete.exclude_deleted/1 schema &quot;users&quot; do field(:is_deleted, :boolean) end end When we want to query without the default scoping applied, we can do so with unscoped/0, which is added by the rescope/1 macro. User.unscoped()"},{"ref":"readme.html#caveats","title":"Ecto.Rescope - Caveats","type":"extras","doc":"Using with Ecto.Query API When using from or join macros from the Ecto.Query api, the query builder defines the queryable source at runtime, thus ignoring __schema__(:query) and the associated override. Because of this, queries such as the following will not work as expected: from(q in User, join: t in Team, on: t.id == q.team_id) # SELECT u0.&quot;id&quot;, u0.&quot;name&quot;, u0.&quot;is_deleted&quot;, u0.&quot;team_id&quot; FROM &quot;users&quot; AS u0 INNER JOIN &quot;teams&quot; AS t1 ON # t1.&quot;id&quot; = u0.&quot;team_id&quot; [] Note the lack of (u0.&quot;is_deleted&quot; = FALSE) or (t1.&quot;is_deleted&quot; = FALSE) in the associated SQL log. However, this can be worked around using the scoped/0 function that is defined by the rescope/0 macro. from(q in User.scoped(), join: t in ^Team.scoped(), on: t.id == q.team_id) # SELECT u0.&quot;id&quot;, u0.&quot;name&quot;, u0.&quot;is_deleted&quot;, u0.&quot;team_id&quot; FROM &quot;users&quot; AS u0 INNER JOIN &quot;teams&quot; AS t1 ON # (t1.&quot;is_deleted&quot; = FALSE) AND (t1.&quot;id&quot; = u0.&quot;team_id&quot;) WHERE (u0.&quot;is_deleted&quot; = FALSE) [] NOTE: Although this can seem a bit cumbersome, when using query building libraries such as DockYard&#39;s Inquisitor, the problem is avoided as the queryable module is converted to a query struct prior to being used within the from macro. Private API This library overrides private reflection functions defined on schemas by the ecto library, specifically __schema__(:query) (source). While this technique has been used stable in production for multiple years, there is no guarantee ecto won&#39;t change the underlying functionality at some point in the future."},{"ref":"readme.html#installation","title":"Ecto.Rescope - Installation","type":"extras","doc":"The package can be installed by adding ecto_rescope to your list of dependencies in mix.exs: def deps do [ {:ecto_rescope, &quot;~&gt; 0.1.0&quot;} ] end Documentation can be found at https://hexdocs.pm/ecto_rescope."},{"ref":"readme.html#license","title":"Ecto.Rescope - License","type":"extras","doc":"The source code is under the Apache 2 License. Copyright (c) 2019 Erik Reedstrom Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License."}]